@@ -23,30 +23,11 @@ char *
 Perl_scan_version(pTHX_ char *s, SV *rv)
 {
     char *pos = s;
-    I32 saw_period = 0;
-    bool saw_under = 0;
+    bool saw_period = 0;
+    bool saw_under  = 0;
     SV* sv = newSVrv(rv, "version"); /* create an SV and upgrade the RV */
     (void)sv_upgrade(sv, SVt_PVAV); /* needs to be an AV type */
 
-    /* pre-scan the imput string to check for decimals */
-    while ( *pos == '.' || *pos == '_' || isDIGIT(*pos) )
-    {
-	if ( *pos == '.' )
-	{
-	    if ( saw_under )
-		croak(aTHX_ "Invalid version format (underscores before decimal)");
-	    saw_period++ ;
-	}
-	else if ( *pos == '_' )
-	{
-	    if ( saw_under )
-		croak(aTHX_ "Invalid version format (multiple underscores)");
-	    saw_under = 1;
-	}
-	pos++;
-    }
-    pos = s;
-
     if (*pos == 'v') pos++;  /* get past 'v' */
     while (isDIGIT(*pos))
     pos++;
@@ -81,17 +62,24 @@ Perl_scan_version(pTHX_ char *s, SV *rv)
 	    /* Append revision */
 	    av_push((AV *)sv, newSViv(rev));
 	    if ( (*pos == '.' || *pos == '_') && isDIGIT(pos[1]))
-		s = ++pos;
-	    else if ( isDIGIT(*pos) )
-		s = pos;
+		 s = ++pos;
 	    else {
-		s = pos;
-		break;
+		 s = pos;
+		 break;
 	    }
-	    while ( isDIGIT(*pos) ) {
-		if ( saw_period == 1 && pos-s == 3 )
-		    break;
-		pos++;
+	    while ( isDIGIT(*pos) )
+		 pos++;
+	    if ( *pos == '.' )
+	    {
+		if ( saw_under )
+		    croak(aTHX_ "Invalid version format (underscores before decimal)");
+		saw_period = 1;
+	    }
+	    else if ( *pos == '_' )
+	    {
+		if ( saw_under )
+		    croak(aTHX_ "Invalid version format (multiple underscores)");
+		saw_under = 1;
 	    }
 	}
     }
@@ -157,26 +145,23 @@ Perl_upg_version(pTHX_ SV *ver)
 /*
 =for apidoc vnumify
 
-Accepts a version object and returns the normalized floating
-point representation.  Call like:
+Accepts a version (or vstring) object and returns the
+normalized floating point representation.  Call like:
 
-    sv = vnumify(rv);
+    sv = vnumify(sv,SvRV(rv));
 
-NOTE: you can pass either the object directly or the SV
-contained within the RV.
+NOTE: no checking is done to see if the object is of the
+correct type (for speed).
 
 =cut
 */
 
 SV *
-Perl_vnumify(pTHX_ SV *vs)
+Perl_vnumify(pTHX_ SV *sv, SV *vs)
 {
-    I32 i, len, digit;
-    SV *sv = NEWSV(92,0);
-    if ( SvROK(vs) )
-	vs = SvRV(vs);
-    len = av_len((AV *)vs);
-    digit = SvIVX(*av_fetch((AV *)vs, 0, 0));
+    I32 i;
+    I32 len = av_len((AV *)vs);
+    I32 digit = SvIVX(*av_fetch((AV *)vs, 0, 0));
     Perl_sv_setpvf(aTHX_ sv,"%d.",abs(digit));
     for ( i = 1 ; i <= len ; i++ )
     {
@@ -191,26 +176,23 @@ Perl_vnumify(pTHX_ SV *vs)
 /*
 =for apidoc vstringify
 
-Accepts a version object and returns the normalized string
-representation.  Call like:
+Accepts a version (or vstring) object and returns the
+normalized representation.  Call like:
 
-    sv = vstringify(rv);
+    sv = vstringify(sv,SvRV(rv));
 
-NOTE: you can pass either the object directly or the SV
-contained within the RV.
+NOTE: no checking is done to see if the object is of the
+correct type (for speed).
 
 =cut
 */
 
 SV *
-Perl_vstringify(pTHX_ SV *vs)
+Perl_vstringify(pTHX_ SV *sv, SV *vs)
 {
-    I32 i, len, digit;
-    SV *sv = NEWSV(92,0);
-    if ( SvROK(vs) )
-	vs = SvRV(vs);
-    len = av_len((AV *)vs);
-    digit = SvIVX(*av_fetch((AV *)vs, 0, 0));
+    I32 i;
+    I32 len = av_len((AV *)vs);
+    I32 digit = SvIVX(*av_fetch((AV *)vs, 0, 0));
     Perl_sv_setpvf(aTHX_ sv,"%d",digit);
     for ( i = 1 ; i <= len ; i++ )
     {
@@ -229,7 +211,8 @@ Perl_vstringify(pTHX_ SV *vs)
 =for apidoc vcmp
 
 Version object aware cmp.  Both operands must already have been 
-converted into version objects.
+converted into version objects and passed as SV's already extracted
+from the RV object.  XS_version_vcmp takes care of this already.
 
 =cut
 */
@@ -237,16 +220,11 @@ converted into version objects.
 int
 Perl_vcmp(pTHX_ SV *lsv, SV *rsv)
 {
-    I32 i,l,m,r,retval;
-    if ( SvROK(lsv) )
-	lsv = SvRV(lsv);
-    if ( SvROK(rsv) )
-	rsv = SvRV(rsv);
-    l = av_len((AV *)lsv);
-    r = av_len((AV *)rsv);
-    m = l < r ? l : r;
-    retval = 0;
-    i = 0;
+    I32 l = av_len((AV *)lsv);
+    I32 r = av_len((AV *)rsv);
+    I32 m = l < r ? l : r;
+    I32 retval = 0;
+    I32 i = 0;
     while ( i <= m && retval == 0 )
     {
 	I32 left  = SvIV(*av_fetch((AV *)lsv,i,0));
